# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: yocto_display.py 66072 2025-04-30 06:59:12Z mvuilleu $
#
#  Implements the asyncio YDisplay API for Display functions
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YDisplay and YDisplayLayer
version: PATCH_WITH_VERSION
requires: yocto_api
requires: yocto_display_aio
"""
from __future__ import annotations
import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    from enum import IntEnum
    from .yocto_api import const, _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True
    _DYNAMIC_HELPERS: Final[bool] = True

from .yocto_display_aio import (
    YDisplay as YDisplay_aio,
    YDisplayLayer as YDisplayLayer_aio
)
from .yocto_api import (
    YAPIContext, YAPI, YFunction, YSyncProxy
)

# --- (generated code: YDisplayLayer class start)
# noinspection PyProtectedMember
class YDisplayLayer(YSyncProxy):
    """
    Each DisplayLayer represents an image layer containing objects
    to display (bitmaps, text, etc.). The content is displayed only when
    the layer is active on the screen (and not masked by other
    overlapping layers).

    """
    _aio: YDisplayLayer_aio
    # --- (end of generated code: YDisplayLayer class start)

    if not _IS_MICROPYTHON:
        # --- (generated code: YDisplayLayer return codes)
        class ALIGN(IntEnum):
            TOP_LEFT = 0
            CENTER_LEFT = 1
            BASELINE_LEFT = 2
            BOTTOM_LEFT = 3
            TOP_CENTER = 4
            CENTER = 5
            BASELINE_CENTER = 6
            BOTTOM_CENTER = 7
            TOP_DECIMAL = 8
            CENTER_DECIMAL = 9
            BASELINE_DECIMAL = 10
            BOTTOM_DECIMAL = 11
            TOP_RIGHT = 12
            CENTER_RIGHT = 13
            BASELINE_RIGHT = 14
            BOTTOM_RIGHT = 15

        # --- (end of generated code: YDisplayLayer return codes)

    # --- (generated code: YDisplayLayer implementation)
    if not _DYNAMIC_HELPERS:
        def reset(self) -> int:
            """
            Reverts the layer to its initial state (fully transparent, default settings).
            Reinitializes the drawing pointer to the upper left position,
            and selects the most visible pen color. If you only want to erase the layer
            content, use the method clear() instead.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.reset())

    if not _DYNAMIC_HELPERS:
        def clear(self) -> int:
            """
            Erases the whole content of the layer (makes it fully transparent).
            This method does not change any other attribute of the layer.
            To reinitialize the layer attributes to defaults settings, use the method
            reset() instead.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.clear())

    if not _DYNAMIC_HELPERS:
        def selectColorPen(self, color: int) -> int:
            """
            Selects the pen color for all subsequent drawing functions,
            including text drawing. The pen color is provided as an RGB value.
            For grayscale or monochrome displays, the value is
            automatically converted to the proper range.

            @param color : the desired pen color, as a 24-bit RGB value

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.selectColorPen(color))

    if not _DYNAMIC_HELPERS:
        def selectGrayPen(self, graylevel: int) -> int:
            """
            Selects the pen gray level for all subsequent drawing functions,
            including text drawing. The gray level is provided as a number between
            0 (black) and 255 (white, or whichever the lightest color is).
            For monochrome displays (without gray levels), any value
            lower than 128 is rendered as black, and any value equal
            or above to 128 is non-black.

            @param graylevel : the desired gray level, from 0 to 255

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.selectGrayPen(graylevel))

    if not _DYNAMIC_HELPERS:
        def selectEraser(self) -> int:
            """
            Selects an eraser instead of a pen for all subsequent drawing functions,
            except for bitmap copy functions. Any point drawn using the eraser
            becomes transparent (as when the layer is empty), showing the other
            layers beneath it.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.selectEraser())

    if not _DYNAMIC_HELPERS:
        def setAntialiasingMode(self, mode: bool) -> int:
            """
            Enables or disables anti-aliasing for drawing oblique lines and circles.
            Anti-aliasing provides a smoother aspect when looked from far enough,
            but it can add fuzziness when the display is looked from very close.
            At the end of the day, it is your personal choice.
            Anti-aliasing is enabled by default on grayscale and color displays,
            but you can disable it if you prefer. This setting has no effect
            on monochrome displays.

            @param mode : true to enable anti-aliasing, false to
                    disable it.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.setAntialiasingMode(mode))

    if not _DYNAMIC_HELPERS:
        def drawPixel(self, x: int, y: int) -> int:
            """
            Draws a single pixel at the specified position.

            @param x : the distance from left of layer, in pixels
            @param y : the distance from top of layer, in pixels

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.drawPixel(x, y))

    if not _DYNAMIC_HELPERS:
        def drawRect(self, x1: int, y1: int, x2: int, y2: int) -> int:
            """
            Draws an empty rectangle at a specified position.

            @param x1 : the distance from left of layer to the left border of the rectangle, in pixels
            @param y1 : the distance from top of layer to the top border of the rectangle, in pixels
            @param x2 : the distance from left of layer to the right border of the rectangle, in pixels
            @param y2 : the distance from top of layer to the bottom border of the rectangle, in pixels

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.drawRect(x1, y1, x2, y2))

    if not _DYNAMIC_HELPERS:
        def drawBar(self, x1: int, y1: int, x2: int, y2: int) -> int:
            """
            Draws a filled rectangular bar at a specified position.

            @param x1 : the distance from left of layer to the left border of the rectangle, in pixels
            @param y1 : the distance from top of layer to the top border of the rectangle, in pixels
            @param x2 : the distance from left of layer to the right border of the rectangle, in pixels
            @param y2 : the distance from top of layer to the bottom border of the rectangle, in pixels

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.drawBar(x1, y1, x2, y2))

    if not _DYNAMIC_HELPERS:
        def drawCircle(self, x: int, y: int, r: int) -> int:
            """
            Draws an empty circle at a specified position.

            @param x : the distance from left of layer to the center of the circle, in pixels
            @param y : the distance from top of layer to the center of the circle, in pixels
            @param r : the radius of the circle, in pixels

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.drawCircle(x, y, r))

    if not _DYNAMIC_HELPERS:
        def drawDisc(self, x: int, y: int, r: int) -> int:
            """
            Draws a filled disc at a given position.

            @param x : the distance from left of layer to the center of the disc, in pixels
            @param y : the distance from top of layer to the center of the disc, in pixels
            @param r : the radius of the disc, in pixels

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.drawDisc(x, y, r))

    if not _DYNAMIC_HELPERS:
        def selectFont(self, fontname: str) -> int:
            """
            Selects a font to use for the next text drawing functions, by providing the name of the
            font file. You can use a built-in font as well as a font file that you have previously
            uploaded to the device built-in memory. If you experience problems selecting a font
            file, check the device logs for any error message such as missing font file or bad font
            file format.

            @param fontname : the font file name, embedded fonts are 8x8.yfm, Small.yfm, Medium.yfm, Large.yfm
            (not available on Yocto-MiniDisplay).

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.selectFont(fontname))

    if not _DYNAMIC_HELPERS:
        def drawText(self, x: int, y: int, anchor: ALIGN, text: str) -> int:
            """
            Draws a text string at the specified position. The point of the text that is aligned
            to the specified pixel position is called the anchor point, and can be chosen among
            several options. Text is rendered from left to right, without implicit wrapping.

            @param x : the distance from left of layer to the text anchor point, in pixels
            @param y : the distance from top of layer to the text anchor point, in pixels
            @param anchor : the text anchor point, chosen among the YDisplayLayer.ALIGN enumeration:
                    YDisplayLayer.ALIGN.TOP_LEFT,         YDisplayLayer.ALIGN.CENTER_LEFT,
                    YDisplayLayer.ALIGN.BASELINE_LEFT,    YDisplayLayer.ALIGN.BOTTOM_LEFT,
                    YDisplayLayer.ALIGN.TOP_CENTER,       YDisplayLayer.ALIGN.CENTER,
                    YDisplayLayer.ALIGN.BASELINE_CENTER,  YDisplayLayer.ALIGN.BOTTOM_CENTER,
                    YDisplayLayer.ALIGN.TOP_DECIMAL,      YDisplayLayer.ALIGN.CENTER_DECIMAL,
                    YDisplayLayer.ALIGN.BASELINE_DECIMAL, YDisplayLayer.ALIGN.BOTTOM_DECIMAL,
                    YDisplayLayer.ALIGN.TOP_RIGHT,        YDisplayLayer.ALIGN.CENTER_RIGHT,
                    YDisplayLayer.ALIGN.BASELINE_RIGHT,   YDisplayLayer.ALIGN.BOTTOM_RIGHT.
            @param text : the text string to draw

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.drawText(x, y, anchor, text))

    if not _DYNAMIC_HELPERS:
        def drawImage(self, x: int, y: int, imagename: str) -> int:
            """
            Draws a GIF image at the specified position. The GIF image must have been previously
            uploaded to the device built-in memory. If you experience problems using an image
            file, check the device logs for any error message such as missing image file or bad
            image file format.

            @param x : the distance from left of layer to the left of the image, in pixels
            @param y : the distance from top of layer to the top of the image, in pixels
            @param imagename : the GIF file name

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.drawImage(x, y, imagename))

    if not _DYNAMIC_HELPERS:
        def drawBitmap(self, x: int, y: int, w: int, bitmap: xarray, bgcol: int) -> int:
            """
            Draws a bitmap at the specified position. The bitmap is provided as a binary object,
            where each pixel maps to a bit, from left to right and from top to bottom.
            The most significant bit of each byte maps to the leftmost pixel, and the least
            significant bit maps to the rightmost pixel. Bits set to 1 are drawn using the
            layer selected pen color. Bits set to 0 are drawn using the specified background
            gray level, unless -1 is specified, in which case they are not drawn at all
            (as if transparent).

            @param x : the distance from left of layer to the left of the bitmap, in pixels
            @param y : the distance from top of layer to the top of the bitmap, in pixels
            @param w : the width of the bitmap, in pixels
            @param bitmap : a binary object
            @param bgcol : the background gray level to use for zero bits (0 = black,
                    255 = white), or -1 to leave the pixels unchanged

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.drawBitmap(x, y, w, bitmap, bgcol))

    if not _DYNAMIC_HELPERS:
        def moveTo(self, x: int, y: int) -> int:
            """
            Moves the drawing pointer of this layer to the specified position.

            @param x : the distance from left of layer, in pixels
            @param y : the distance from top of layer, in pixels

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.moveTo(x, y))

    if not _DYNAMIC_HELPERS:
        def lineTo(self, x: int, y: int) -> int:
            """
            Draws a line from current drawing pointer position to the specified position.
            The specified destination pixel is included in the line. The pointer position
            is then moved to the end point of the line.

            @param x : the distance from left of layer to the end point of the line, in pixels
            @param y : the distance from top of layer to the end point of the line, in pixels

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.lineTo(x, y))

    if not _DYNAMIC_HELPERS:
        def consoleOut(self, text: str) -> int:
            """
            Outputs a message in the console area, and advances the console pointer accordingly.
            The console pointer position is automatically moved to the beginning
            of the next line when a newline character is met, or when the right margin
            is hit. When the new text to display extends below the lower margin, the
            console area is automatically scrolled up.

            @param text : the message to display

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.consoleOut(text))

    if not _DYNAMIC_HELPERS:
        def setConsoleMargins(self, x1: int, y1: int, x2: int, y2: int) -> int:
            """
            Sets up display margins for the consoleOut function.

            @param x1 : the distance from left of layer to the left margin, in pixels
            @param y1 : the distance from top of layer to the top margin, in pixels
            @param x2 : the distance from left of layer to the right margin, in pixels
            @param y2 : the distance from top of layer to the bottom margin, in pixels

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.setConsoleMargins(x1, y1, x2, y2))

    if not _DYNAMIC_HELPERS:
        def setConsoleBackground(self, bgcol: int) -> int:
            """
            Sets up the background color used by the clearConsole function and by
            the console scrolling feature.

            @param bgcol : the background gray level to use when scrolling (0 = black,
                    255 = white), or -1 for transparent

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.setConsoleBackground(bgcol))

    if not _DYNAMIC_HELPERS:
        def setConsoleWordWrap(self, wordwrap: bool) -> int:
            """
            Sets up the wrapping behavior used by the consoleOut function.

            @param wordwrap : true to wrap only between words,
                    false to wrap on the last column anyway.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.setConsoleWordWrap(wordwrap))

    if not _DYNAMIC_HELPERS:
        def clearConsole(self) -> int:
            """
            Blanks the console area within console margins, and resets the console pointer
            to the upper left corner of the console.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.clearConsole())

    if not _DYNAMIC_HELPERS:
        def setLayerPosition(self, x: int, y: int, scrollTime: int) -> int:
            """
            Sets the position of the layer relative to the display upper left corner.
            When smooth scrolling is used, the display offset of the layer is
            automatically updated during the next milliseconds to animate the move of the layer.

            @param x : the distance from left of display to the upper left corner of the layer
            @param y : the distance from top of display to the upper left corner of the layer
            @param scrollTime : number of milliseconds to use for smooth scrolling, or
                    0 if the scrolling should be immediate.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.setLayerPosition(x, y, scrollTime))

    if not _DYNAMIC_HELPERS:
        def hide(self) -> int:
            """
            Hides the layer. The state of the layer is preserved but the layer is not displayed
            on the screen until the next call to unhide(). Hiding the layer can positively
            affect the drawing speed, since it postpones the rendering until all operations are
            completed (double-buffering).

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.hide())

    if not _DYNAMIC_HELPERS:
        def unhide(self) -> int:
            """
            Shows the layer. Shows the layer again after a hide command.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.unhide())

    def get_display(self) -> YDisplay:
        """
        Gets parent YDisplay. Returns the parent YDisplay object of the current YDisplayLayer.

        @return an YDisplay object
        """
        return self._proxy(YDisplay, self._run(self._aio.get_display()))

    if not _DYNAMIC_HELPERS:
        def get_displayWidth(self) -> int:
            """
            Returns the display width, in pixels.

            @return an integer corresponding to the display width, in pixels

            On failure, throws an exception or returns YDisplayLayer.DISPLAYWIDTH_INVALID.
            """
            return self._run(self._aio.get_displayWidth())

    if not _DYNAMIC_HELPERS:
        def get_displayHeight(self) -> int:
            """
            Returns the display height, in pixels.

            @return an integer corresponding to the display height, in pixels

            On failure, throws an exception or returns YDisplayLayer.DISPLAYHEIGHT_INVALID.
            """
            return self._run(self._aio.get_displayHeight())

    if not _DYNAMIC_HELPERS:
        def get_layerWidth(self) -> int:
            """
            Returns the width of the layers to draw on, in pixels.

            @return an integer corresponding to the width of the layers to draw on, in pixels

            On failure, throws an exception or returns YDisplayLayer.LAYERWIDTH_INVALID.
            """
            return self._run(self._aio.get_layerWidth())

    if not _DYNAMIC_HELPERS:
        def get_layerHeight(self) -> int:
            """
            Returns the height of the layers to draw on, in pixels.

            @return an integer corresponding to the height of the layers to draw on, in pixels

            On failure, throws an exception or returns YDisplayLayer.LAYERHEIGHT_INVALID.
            """
            return self._run(self._aio.get_layerHeight())

    # --- (end of generated code: YDisplayLayer implementation)

# --- (generated code: YDisplay class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YDisplayValueCallback = Union[Callable[['YDisplay', str], Awaitable[None]], None]
    except TypeError:
        YDisplayValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YDisplay(YFunction):
    """
    The YDisplay class allows to drive Yoctopuce displays.
    Yoctopuce display interface has been designed to easily
    show information and images. The device provides built-in
    multi-layer rendering. Layers can be drawn offline, individually,
    and freely moved on the display. It can also replay recorded
    sequences (animations).

    In order to draw on the screen, you should use the
    display.get_displayLayer method to retrieve the layer(s) on
    which you want to draw, and then use methods defined in
    YDisplayLayer to draw on the layers.

    """
    _aio: YDisplay_aio
    # --- (end of generated code: YDisplay class start)
    if not _IS_MICROPYTHON:
        # --- (generated code: YDisplay return codes)
        STARTUPSEQ_INVALID: Final[str] = YAPI.INVALID_STRING
        BRIGHTNESS_INVALID: Final[int] = YAPI.INVALID_UINT
        DISPLAYWIDTH_INVALID: Final[int] = YAPI.INVALID_UINT
        DISPLAYHEIGHT_INVALID: Final[int] = YAPI.INVALID_UINT
        LAYERWIDTH_INVALID: Final[int] = YAPI.INVALID_UINT
        LAYERHEIGHT_INVALID: Final[int] = YAPI.INVALID_UINT
        LAYERCOUNT_INVALID: Final[int] = YAPI.INVALID_UINT
        COMMAND_INVALID: Final[str] = YAPI.INVALID_STRING
        ENABLED_FALSE: Final[int] = 0
        ENABLED_TRUE: Final[int] = 1
        ENABLED_INVALID: Final[int] = -1
        ORIENTATION_LEFT: Final[int] = 0
        ORIENTATION_UP: Final[int] = 1
        ORIENTATION_RIGHT: Final[int] = 2
        ORIENTATION_DOWN: Final[int] = 3
        ORIENTATION_INVALID: Final[int] = -1
        DISPLAYTYPE_MONO: Final[int] = 0
        DISPLAYTYPE_GRAY: Final[int] = 1
        DISPLAYTYPE_RGB: Final[int] = 2
        DISPLAYTYPE_INVALID: Final[int] = -1
        # --- (end of generated code: YDisplay return codes)

    # --- (generated code: YDisplay attributes declaration)
    _enabled: int
    _startupSeq: str
    _brightness: int
    _orientation: int
    _displayWidth: int
    _displayHeight: int
    _displayType: int
    _layerWidth: int
    _layerHeight: int
    _layerCount: int
    _command: str
    _valueCallbackDisplay: YDisplayValueCallback
    _allDisplayLayers: list[YDisplayLayer]
    # --- (end of generated code: YDisplay attributes declaration)

    # --- (generated code: YDisplay implementation)

    @classmethod
    def FirstDisplay(cls) -> Union[YDisplay, None]:
        """
        Starts the enumeration of displays currently accessible.
        Use the method YDisplay.nextDisplay() to iterate on
        next displays.

        @return a pointer to a YDisplay object, corresponding to
                the first display currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YDisplay_aio.FirstDisplay())

    @classmethod
    def FirstDisplayInContext(cls, yctx: YAPIContext) -> Union[YDisplay, None]:
        """
        Starts the enumeration of displays currently accessible.
        Use the method YDisplay.nextDisplay() to iterate on
        next displays.

        @param yctx : a YAPI context.

        @return a pointer to a YDisplay object, corresponding to
                the first display currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YDisplay_aio.FirstDisplayInContext(yctx))

    def nextDisplay(self):
        """
        Continues the enumeration of displays started using yFirstDisplay().
        Caution: You can't make any assumption about the returned displays order.
        If you want to find a specific a display, use Display.findDisplay()
        and a hardwareID or a logical name.

        @return a pointer to a YDisplay object, corresponding to
                a display currently online, or a None pointer
                if there are no more displays to enumerate.
        """
        return self._proxy(type(self), self._aio.nextDisplay())

    if not _DYNAMIC_HELPERS:
        def get_enabled(self) -> int:
            """
            Returns true if the screen is powered, false otherwise.

            @return either YDisplay.ENABLED_FALSE or YDisplay.ENABLED_TRUE, according to true if the screen is
            powered, false otherwise

            On failure, throws an exception or returns YDisplay.ENABLED_INVALID.
            """
            return self._run(self._aio.get_enabled())

    if not _DYNAMIC_HELPERS:
        def set_enabled(self, newval: int) -> int:
            """
            Changes the power state of the display.

            @param newval : either YDisplay.ENABLED_FALSE or YDisplay.ENABLED_TRUE, according to the power
            state of the display

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_enabled(newval))

    if not _DYNAMIC_HELPERS:
        def get_startupSeq(self) -> str:
            """
            Returns the name of the sequence to play when the displayed is powered on.

            @return a string corresponding to the name of the sequence to play when the displayed is powered on

            On failure, throws an exception or returns YDisplay.STARTUPSEQ_INVALID.
            """
            return self._run(self._aio.get_startupSeq())

    if not _DYNAMIC_HELPERS:
        def set_startupSeq(self, newval: str) -> int:
            """
            Changes the name of the sequence to play when the displayed is powered on.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a string corresponding to the name of the sequence to play when the displayed is powered on

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_startupSeq(newval))

    if not _DYNAMIC_HELPERS:
        def get_brightness(self) -> int:
            """
            Returns the luminosity of the  module informative LEDs (from 0 to 100).

            @return an integer corresponding to the luminosity of the  module informative LEDs (from 0 to 100)

            On failure, throws an exception or returns YDisplay.BRIGHTNESS_INVALID.
            """
            return self._run(self._aio.get_brightness())

    if not _DYNAMIC_HELPERS:
        def set_brightness(self, newval: int) -> int:
            """
            Changes the brightness of the display. The parameter is a value between 0 and
            100. Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : an integer corresponding to the brightness of the display

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_brightness(newval))

    if not _DYNAMIC_HELPERS:
        def get_orientation(self) -> int:
            """
            Returns the currently selected display orientation.

            @return a value among YDisplay.ORIENTATION_LEFT, YDisplay.ORIENTATION_UP,
            YDisplay.ORIENTATION_RIGHT and YDisplay.ORIENTATION_DOWN corresponding to the currently selected
            display orientation

            On failure, throws an exception or returns YDisplay.ORIENTATION_INVALID.
            """
            return self._run(self._aio.get_orientation())

    if not _DYNAMIC_HELPERS:
        def set_orientation(self, newval: int) -> int:
            """
            Changes the display orientation. Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : a value among YDisplay.ORIENTATION_LEFT, YDisplay.ORIENTATION_UP,
            YDisplay.ORIENTATION_RIGHT and YDisplay.ORIENTATION_DOWN corresponding to the display orientation

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_orientation(newval))

    if not _DYNAMIC_HELPERS:
        def get_displayWidth(self) -> int:
            """
            Returns the display width, in pixels.

            @return an integer corresponding to the display width, in pixels

            On failure, throws an exception or returns YDisplay.DISPLAYWIDTH_INVALID.
            """
            return self._run(self._aio.get_displayWidth())

    if not _DYNAMIC_HELPERS:
        def get_displayHeight(self) -> int:
            """
            Returns the display height, in pixels.

            @return an integer corresponding to the display height, in pixels

            On failure, throws an exception or returns YDisplay.DISPLAYHEIGHT_INVALID.
            """
            return self._run(self._aio.get_displayHeight())

    if not _DYNAMIC_HELPERS:
        def get_displayType(self) -> int:
            """
            Returns the display type: monochrome, gray levels or full color.

            @return a value among YDisplay.DISPLAYTYPE_MONO, YDisplay.DISPLAYTYPE_GRAY and
            YDisplay.DISPLAYTYPE_RGB corresponding to the display type: monochrome, gray levels or full color

            On failure, throws an exception or returns YDisplay.DISPLAYTYPE_INVALID.
            """
            return self._run(self._aio.get_displayType())

    if not _DYNAMIC_HELPERS:
        def get_layerWidth(self) -> int:
            """
            Returns the width of the layers to draw on, in pixels.

            @return an integer corresponding to the width of the layers to draw on, in pixels

            On failure, throws an exception or returns YDisplay.LAYERWIDTH_INVALID.
            """
            return self._run(self._aio.get_layerWidth())

    if not _DYNAMIC_HELPERS:
        def get_layerHeight(self) -> int:
            """
            Returns the height of the layers to draw on, in pixels.

            @return an integer corresponding to the height of the layers to draw on, in pixels

            On failure, throws an exception or returns YDisplay.LAYERHEIGHT_INVALID.
            """
            return self._run(self._aio.get_layerHeight())

    if not _DYNAMIC_HELPERS:
        def get_layerCount(self) -> int:
            """
            Returns the number of available layers to draw on.

            @return an integer corresponding to the number of available layers to draw on

            On failure, throws an exception or returns YDisplay.LAYERCOUNT_INVALID.
            """
            return self._run(self._aio.get_layerCount())

    if not _DYNAMIC_HELPERS:
        def set_command(self, newval: str) -> int:
            return self._run(self._aio.set_command(newval))

    @classmethod
    def FindDisplay(cls, func: str) -> YDisplay:
        """
        Retrieves a display for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the display is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YDisplay.isOnline() to test if the display is
        indeed online at a given time. In case of ambiguity when looking for
        a display by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the display, for instance
                YD128X32.display.

        @return a YDisplay object allowing you to drive the display.
        """
        return cls._proxy(cls, YDisplay_aio.FindDisplay(func))

    @classmethod
    def FindDisplayInContext(cls, yctx: YAPIContext, func: str) -> YDisplay:
        """
        Retrieves a display for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the display is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YDisplay.isOnline() to test if the display is
        indeed online at a given time. In case of ambiguity when looking for
        a display by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the display, for instance
                YD128X32.display.

        @return a YDisplay object allowing you to drive the display.
        """
        return cls._proxy(cls, YDisplay_aio.FindDisplayInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YDisplayValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _DYNAMIC_HELPERS:
        def resetAll(self) -> int:
            """
            Clears the display screen and resets all display layers to their default state.
            Using this function in a sequence will kill the sequence play-back. Don't use that
            function to reset the display at sequence start-up.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.resetAll())

    if not _DYNAMIC_HELPERS:
        def fade(self, brightness: int, duration: int) -> int:
            """
            Smoothly changes the brightness of the screen to produce a fade-in or fade-out
            effect.

            @param brightness : the new screen brightness
            @param duration : duration of the brightness transition, in milliseconds.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.fade(brightness, duration))

    if not _DYNAMIC_HELPERS:
        def newSequence(self) -> int:
            """
            Starts to record all display commands into a sequence, for later replay.
            The name used to store the sequence is specified when calling
            saveSequence(), once the recording is complete.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.newSequence())

    if not _DYNAMIC_HELPERS:
        def saveSequence(self, sequenceName: str) -> int:
            """
            Stops recording display commands and saves the sequence into the specified
            file on the display internal memory. The sequence can be later replayed
            using playSequence().

            @param sequenceName : the name of the newly created sequence

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.saveSequence(sequenceName))

    if not _DYNAMIC_HELPERS:
        def playSequence(self, sequenceName: str) -> int:
            """
            Replays a display sequence previously recorded using
            newSequence() and saveSequence().

            @param sequenceName : the name of the newly created sequence

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.playSequence(sequenceName))

    if not _DYNAMIC_HELPERS:
        def pauseSequence(self, delay_ms: int) -> int:
            """
            Waits for a specified delay (in milliseconds) before playing next
            commands in current sequence. This method can be used while
            recording a display sequence, to insert a timed wait in the sequence
            (without any immediate effect). It can also be used dynamically while
            playing a pre-recorded sequence, to suspend or resume the execution of
            the sequence. To cancel a delay, call the same method with a zero delay.

            @param delay_ms : the duration to wait, in milliseconds

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.pauseSequence(delay_ms))

    if not _DYNAMIC_HELPERS:
        def stopSequence(self) -> int:
            """
            Stops immediately any ongoing sequence replay.
            The display is left as is.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.stopSequence())

    if not _DYNAMIC_HELPERS:
        def upload(self, pathname: str, content: xarray) -> int:
            """
            Uploads an arbitrary file (for instance a GIF file) to the display, to the
            specified full path name. If a file already exists with the same path name,
            its content is overwritten.

            @param pathname : path and name of the new file to create
            @param content : binary buffer with the content to set

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.upload(pathname, content))

    if not _DYNAMIC_HELPERS:
        def copyLayerContent(self, srcLayerId: int, dstLayerId: int) -> int:
            """
            Copies the whole content of a layer to another layer. The color and transparency
            of all the pixels from the destination layer are set to match the source pixels.
            This method only affects the displayed content, but does not change any
            property of the layer object.
            Note that layer 0 has no transparency support (it is always completely opaque).

            @param srcLayerId : the identifier of the source layer (a number in range 0..layerCount-1)
            @param dstLayerId : the identifier of the destination layer (a number in range 0..layerCount-1)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.copyLayerContent(srcLayerId, dstLayerId))

    if not _DYNAMIC_HELPERS:
        def swapLayerContent(self, layerIdA: int, layerIdB: int) -> int:
            """
            Swaps the whole content of two layers. The color and transparency of all the pixels from
            the two layers are swapped. This method only affects the displayed content, but does
            not change any property of the layer objects. In particular, the visibility of each
            layer stays unchanged. When used between one hidden layer and a visible layer,
            this method makes it possible to easily implement double-buffering.
            Note that layer 0 has no transparency support (it is always completely opaque).

            @param layerIdA : the first layer (a number in range 0..layerCount-1)
            @param layerIdB : the second layer (a number in range 0..layerCount-1)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.swapLayerContent(layerIdA, layerIdB))

    def get_displayLayer(self, layerId: int) -> Union[YDisplayLayer, None]:
        """
        Returns a YDisplayLayer object that can be used to draw on the specified
        layer. The content is displayed only when the layer is active on the
        screen (and not masked by other overlapping layers).

        @param layerId : the identifier of the layer (a number in range 0..layerCount-1)

        @return an YDisplayLayer object

        On failure, throws an exception or returns None.
        """
        return self._proxy(YDisplayLayer, self._run(self._aio.get_displayLayer(layerId)))

    # --- (end of generated code: YDisplay implementation)




# ********************************************************************
#
#  $Id: helloworld.py 45344 2021-05-28 09:44:39Z web $
#
#  An example that show how to use a  Yocto-MaxiDisplay
#
#  You can find more information on our web site:
#   Yocto-MaxiDisplay documentation:
#      https://www.yoctopuce.com/EN/products/yocto-maxidisplay/doc.html
#   Python V2 API Reference:
#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-python-EN.html
#
# *********************************************************************

#!/usr/bin/python
# -*- coding: utf-8 -*-
import os, sys
# add ../../Sources to the PYTHONPATH
sys.path.append(os.path.join("..", "..", "Sources"))

from yocto_api import *
from yocto_display import *


def usage():
    scriptname = os.path.basename(sys.argv[0])
    print("Usage:")
    print(scriptname + ' <serial_number>')
    print(scriptname + ' <logical_name>')
    print(scriptname + ' any  ')
    sys.exit()


def die(msg):
    sys.exit(msg + ' (check USB cable)')


errmsg = YRefParam()

if len(sys.argv) < 2:
    usage()

target = sys.argv[1]

# Setup the API to use local USB devices
if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
    sys.exit("init error" + errmsg.value)

if target == 'any':
    # retreive any RGB led
    disp = YDisplay.FirstDisplay()
    if disp is None:
        die('No module connected')
else:
    disp = YDisplay.FindDisplay(target + ".display")

if not disp.isOnline():
    die("Module not connected ")

# display clean up
disp.resetAll()

# retreive the display size
w = disp.get_displayWidth()
h = disp.get_displayHeight()

# retreive the first layer
l0 = disp.get_displayLayer(0)
l0.clear()

# display a text in the middle of the screen
l0.drawText(w / 2, h / 2, YDisplayLayer.ALIGN.CENTER, "Hello world!")

# visualize each corner
l0.moveTo(0, 5)
l0.lineTo(0, 0)
l0.lineTo(5, 0)
l0.moveTo(0, h - 6)
l0.lineTo(0, h - 1)
l0.lineTo(5, h - 1)
l0.moveTo(w - 1, h - 6)
l0.lineTo(w - 1, h - 1)
l0.lineTo(w - 6, h - 1)
l0.moveTo(w - 1, 5)
l0.lineTo(w - 1, 0)
l0.lineTo(w - 6, 0)

# draw a circle in the top left corner of layer 1
l1 = disp.get_displayLayer(1)
l1.clear()
l1.drawCircle(h / 8, h / 8, h / 8)

# and animate the layer
print("Use Ctrl-C to stop")
x = 0
y = 0
vx = 1
vy = 1
while disp.isOnline():
    x += vx
    y += vy
    if x < 0 or x > w - (h / 4):
        vx = -vx
    if y < 0 or y > h - (h / 4):
        vy = -vy
    l1.setLayerPosition(x, y, 0)
    YAPI.Sleep(5, errmsg)
YAPI.FreeAPI()



# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: yocto_i2cport.py 66072 2025-04-30 06:59:12Z mvuilleu $
#
#  Implements the asyncio YI2cPort API for I2cPort functions
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YI2cPort
version: PATCH_WITH_VERSION
requires: yocto_api
requires: yocto_i2cport_aio
"""
from __future__ import annotations
import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import const, _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True
    _DYNAMIC_HELPERS: Final[bool] = True

from .yocto_i2cport_aio import (
    YI2cPort as YI2cPort_aio,
    YI2cSnoopingRecord
)
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)


# --- (generated code: YI2cPort class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YI2cPortValueCallback = Union[Callable[['YI2cPort', str], Awaitable[None]], None]
    except TypeError:
        YI2cPortValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YI2cPort(YFunction):
    """
    The YI2cPort classe allows you to fully drive a Yoctopuce I2C port.
    It can be used to send and receive data, and to configure communication
    parameters (baud rate, etc).
    Note that Yoctopuce I2C ports are not exposed as virtual COM ports.
    They are meant to be used in the same way as all Yoctopuce devices.

    """
    _aio: YI2cPort_aio
    # --- (end of generated code: YI2cPort class start)
    if not _IS_MICROPYTHON:
        # --- (generated code: YI2cPort return codes)
        RXCOUNT_INVALID: Final[int] = YAPI.INVALID_UINT
        TXCOUNT_INVALID: Final[int] = YAPI.INVALID_UINT
        ERRCOUNT_INVALID: Final[int] = YAPI.INVALID_UINT
        RXMSGCOUNT_INVALID: Final[int] = YAPI.INVALID_UINT
        TXMSGCOUNT_INVALID: Final[int] = YAPI.INVALID_UINT
        LASTMSG_INVALID: Final[str] = YAPI.INVALID_STRING
        CURRENTJOB_INVALID: Final[str] = YAPI.INVALID_STRING
        STARTUPJOB_INVALID: Final[str] = YAPI.INVALID_STRING
        JOBMAXTASK_INVALID: Final[int] = YAPI.INVALID_UINT
        JOBMAXSIZE_INVALID: Final[int] = YAPI.INVALID_UINT
        COMMAND_INVALID: Final[str] = YAPI.INVALID_STRING
        PROTOCOL_INVALID: Final[str] = YAPI.INVALID_STRING
        I2CMODE_INVALID: Final[str] = YAPI.INVALID_STRING
        I2CVOLTAGELEVEL_OFF: Final[int] = 0
        I2CVOLTAGELEVEL_3V3: Final[int] = 1
        I2CVOLTAGELEVEL_1V8: Final[int] = 2
        I2CVOLTAGELEVEL_INVALID: Final[int] = -1
        # --- (end of generated code: YI2cPort return codes)

    # --- (generated code: YI2cPort attributes declaration)
    _rxCount: int
    _txCount: int
    _errCount: int
    _rxMsgCount: int
    _txMsgCount: int
    _lastMsg: str
    _currentJob: str
    _startupJob: str
    _jobMaxTask: int
    _jobMaxSize: int
    _command: str
    _protocol: str
    _i2cVoltageLevel: int
    _i2cMode: str
    _valueCallbackI2cPort: YI2cPortValueCallback
    _rxptr: int
    _rxbuff: xarray
    _rxbuffptr: int
    # --- (end of generated code: YI2cPort attributes declaration)

    # --- (generated code: YI2cPort implementation)

    @classmethod
    def FirstI2cPort(cls) -> Union[YI2cPort, None]:
        """
        Starts the enumeration of I2C ports currently accessible.
        Use the method YI2cPort.nextI2cPort() to iterate on
        next I2C ports.

        @return a pointer to a YI2cPort object, corresponding to
                the first I2C port currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YI2cPort_aio.FirstI2cPort())

    @classmethod
    def FirstI2cPortInContext(cls, yctx: YAPIContext) -> Union[YI2cPort, None]:
        """
        Starts the enumeration of I2C ports currently accessible.
        Use the method YI2cPort.nextI2cPort() to iterate on
        next I2C ports.

        @param yctx : a YAPI context.

        @return a pointer to a YI2cPort object, corresponding to
                the first I2C port currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YI2cPort_aio.FirstI2cPortInContext(yctx))

    def nextI2cPort(self):
        """
        Continues the enumeration of I2C ports started using yFirstI2cPort().
        Caution: You can't make any assumption about the returned I2C ports order.
        If you want to find a specific an I2C port, use I2cPort.findI2cPort()
        and a hardwareID or a logical name.

        @return a pointer to a YI2cPort object, corresponding to
                an I2C port currently online, or a None pointer
                if there are no more I2C ports to enumerate.
        """
        return self._proxy(type(self), self._aio.nextI2cPort())

    if not _DYNAMIC_HELPERS:
        def get_rxCount(self) -> int:
            """
            Returns the total number of bytes received since last reset.

            @return an integer corresponding to the total number of bytes received since last reset

            On failure, throws an exception or returns YI2cPort.RXCOUNT_INVALID.
            """
            return self._run(self._aio.get_rxCount())

    if not _DYNAMIC_HELPERS:
        def get_txCount(self) -> int:
            """
            Returns the total number of bytes transmitted since last reset.

            @return an integer corresponding to the total number of bytes transmitted since last reset

            On failure, throws an exception or returns YI2cPort.TXCOUNT_INVALID.
            """
            return self._run(self._aio.get_txCount())

    if not _DYNAMIC_HELPERS:
        def get_errCount(self) -> int:
            """
            Returns the total number of communication errors detected since last reset.

            @return an integer corresponding to the total number of communication errors detected since last reset

            On failure, throws an exception or returns YI2cPort.ERRCOUNT_INVALID.
            """
            return self._run(self._aio.get_errCount())

    if not _DYNAMIC_HELPERS:
        def get_rxMsgCount(self) -> int:
            """
            Returns the total number of messages received since last reset.

            @return an integer corresponding to the total number of messages received since last reset

            On failure, throws an exception or returns YI2cPort.RXMSGCOUNT_INVALID.
            """
            return self._run(self._aio.get_rxMsgCount())

    if not _DYNAMIC_HELPERS:
        def get_txMsgCount(self) -> int:
            """
            Returns the total number of messages send since last reset.

            @return an integer corresponding to the total number of messages send since last reset

            On failure, throws an exception or returns YI2cPort.TXMSGCOUNT_INVALID.
            """
            return self._run(self._aio.get_txMsgCount())

    if not _DYNAMIC_HELPERS:
        def get_lastMsg(self) -> str:
            """
            Returns the latest message fully received (for Line and Frame protocols).

            @return a string corresponding to the latest message fully received (for Line and Frame protocols)

            On failure, throws an exception or returns YI2cPort.LASTMSG_INVALID.
            """
            return self._run(self._aio.get_lastMsg())

    if not _DYNAMIC_HELPERS:
        def get_currentJob(self) -> str:
            """
            Returns the name of the job file currently in use.

            @return a string corresponding to the name of the job file currently in use

            On failure, throws an exception or returns YI2cPort.CURRENTJOB_INVALID.
            """
            return self._run(self._aio.get_currentJob())

    if not _DYNAMIC_HELPERS:
        def set_currentJob(self, newval: str) -> int:
            """
            Selects a job file to run immediately. If an empty string is
            given as argument, stops running current job file.

            @param newval : a string

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_currentJob(newval))

    if not _DYNAMIC_HELPERS:
        def get_startupJob(self) -> str:
            """
            Returns the job file to use when the device is powered on.

            @return a string corresponding to the job file to use when the device is powered on

            On failure, throws an exception or returns YI2cPort.STARTUPJOB_INVALID.
            """
            return self._run(self._aio.get_startupJob())

    if not _DYNAMIC_HELPERS:
        def set_startupJob(self, newval: str) -> int:
            """
            Changes the job to use when the device is powered on.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a string corresponding to the job to use when the device is powered on

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_startupJob(newval))

    if not _DYNAMIC_HELPERS:
        def get_jobMaxTask(self) -> int:
            """
            Returns the maximum number of tasks in a job that the device can handle.

            @return an integer corresponding to the maximum number of tasks in a job that the device can handle

            On failure, throws an exception or returns YI2cPort.JOBMAXTASK_INVALID.
            """
            return self._run(self._aio.get_jobMaxTask())

    if not _DYNAMIC_HELPERS:
        def get_jobMaxSize(self) -> int:
            """
            Returns maximum size allowed for job files.

            @return an integer corresponding to maximum size allowed for job files

            On failure, throws an exception or returns YI2cPort.JOBMAXSIZE_INVALID.
            """
            return self._run(self._aio.get_jobMaxSize())

    if not _DYNAMIC_HELPERS:
        def set_command(self, newval: str) -> int:
            return self._run(self._aio.set_command(newval))

    if not _DYNAMIC_HELPERS:
        def get_protocol(self) -> str:
            """
            Returns the type of protocol used to send I2C messages, as a string.
            Possible values are
            "Line" for messages separated by LF or
            "Char" for continuous stream of codes.

            @return a string corresponding to the type of protocol used to send I2C messages, as a string

            On failure, throws an exception or returns YI2cPort.PROTOCOL_INVALID.
            """
            return self._run(self._aio.get_protocol())

    if not _DYNAMIC_HELPERS:
        def set_protocol(self, newval: str) -> int:
            """
            Changes the type of protocol used to send I2C messages.
            Possible values are
            "Line" for messages separated by LF or
            "Char" for continuous stream of codes.
            The suffix "/[wait]ms" can be added to reduce the transmit rate so that there
            is always at lest the specified number of milliseconds between each message sent.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a string corresponding to the type of protocol used to send I2C messages

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_protocol(newval))

    if not _DYNAMIC_HELPERS:
        def get_i2cVoltageLevel(self) -> int:
            """
            Returns the voltage level used on the I2C bus.

            @return a value among YI2cPort.I2CVOLTAGELEVEL_OFF, YI2cPort.I2CVOLTAGELEVEL_3V3 and
            YI2cPort.I2CVOLTAGELEVEL_1V8 corresponding to the voltage level used on the I2C bus

            On failure, throws an exception or returns YI2cPort.I2CVOLTAGELEVEL_INVALID.
            """
            return self._run(self._aio.get_i2cVoltageLevel())

    if not _DYNAMIC_HELPERS:
        def set_i2cVoltageLevel(self, newval: int) -> int:
            """
            Changes the voltage level used on the I2C bus.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a value among YI2cPort.I2CVOLTAGELEVEL_OFF, YI2cPort.I2CVOLTAGELEVEL_3V3 and
            YI2cPort.I2CVOLTAGELEVEL_1V8 corresponding to the voltage level used on the I2C bus

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_i2cVoltageLevel(newval))

    if not _DYNAMIC_HELPERS:
        def get_i2cMode(self) -> str:
            """
            Returns the I2C port communication parameters, as a string such as
            "400kbps,2000ms,NoRestart". The string includes the baud rate, the
            recovery delay after communications errors, and if needed the option
            NoRestart to use a Stop/Start sequence instead of the
            Restart state when performing read on the I2C bus.

            @return a string corresponding to the I2C port communication parameters, as a string such as
                    "400kbps,2000ms,NoRestart"

            On failure, throws an exception or returns YI2cPort.I2CMODE_INVALID.
            """
            return self._run(self._aio.get_i2cMode())

    if not _DYNAMIC_HELPERS:
        def set_i2cMode(self, newval: str) -> int:
            """
            Changes the I2C port communication parameters, with a string such as
            "400kbps,2000ms". The string includes the baud rate, the
            recovery delay after communications errors, and if needed the option
            NoRestart to use a Stop/Start sequence instead of the
            Restart state when performing read on the I2C bus.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a string corresponding to the I2C port communication parameters, with a string such as
                    "400kbps,2000ms"

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_i2cMode(newval))

    @classmethod
    def FindI2cPort(cls, func: str) -> YI2cPort:
        """
        Retrieves an I2C port for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the I2C port is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YI2cPort.isOnline() to test if the I2C port is
        indeed online at a given time. In case of ambiguity when looking for
        an I2C port by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the I2C port, for instance
                YI2CMK01.i2cPort.

        @return a YI2cPort object allowing you to drive the I2C port.
        """
        return cls._proxy(cls, YI2cPort_aio.FindI2cPort(func))

    @classmethod
    def FindI2cPortInContext(cls, yctx: YAPIContext, func: str) -> YI2cPort:
        """
        Retrieves an I2C port for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the I2C port is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YI2cPort.isOnline() to test if the I2C port is
        indeed online at a given time. In case of ambiguity when looking for
        an I2C port by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the I2C port, for instance
                YI2CMK01.i2cPort.

        @return a YI2cPort object allowing you to drive the I2C port.
        """
        return cls._proxy(cls, YI2cPort_aio.FindI2cPortInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YI2cPortValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _DYNAMIC_HELPERS:
        def readLine(self) -> str:
            """
            Reads a single line (or message) from the receive buffer, starting at current stream position.
            This function is intended to be used when the serial port is configured for a message protocol,
            such as 'Line' mode or frame protocols.

            If data at current stream position is not available anymore in the receive buffer,
            the function returns the oldest available line and moves the stream position just after.
            If no new full line is received, the function returns an empty line.

            @return a string with a single line of text

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.readLine())

    if not _DYNAMIC_HELPERS:
        def readMessages(self, pattern: str, maxWait: int) -> list[str]:
            """
            Searches for incoming messages in the serial port receive buffer matching a given pattern,
            starting at current position. This function will only compare and return printable characters
            in the message strings. Binary protocols are handled as hexadecimal strings.

            The search returns all messages matching the expression provided as argument in the buffer.
            If no matching message is found, the search waits for one up to the specified maximum timeout
            (in milliseconds).

            @param pattern : a limited regular expression describing the expected message format,
                    or an empty string if all messages should be returned (no filtering).
                    When using binary protocols, the format applies to the hexadecimal
                    representation of the message.
            @param maxWait : the maximum number of milliseconds to wait for a message if none is found
                    in the receive buffer.

            @return an array of strings containing the messages found, if any.
                    Binary messages are converted to hexadecimal representation.

            On failure, throws an exception or returns an empty array.
            """
            return self._run(self._aio.readMessages(pattern, maxWait))

    if not _DYNAMIC_HELPERS:
        def read_seek(self, absPos: int) -> int:
            """
            Changes the current internal stream position to the specified value. This function
            does not affect the device, it only changes the value stored in the API object
            for the next read operations.

            @param absPos : the absolute position index for next read operations.

            @return nothing.
            """
            return self._run(self._aio.read_seek(absPos))

    if not _DYNAMIC_HELPERS:
        def read_tell(self) -> int:
            """
            Returns the current absolute stream position pointer of the API object.

            @return the absolute position index for next read operations.
            """
            return self._run(self._aio.read_tell())

    if not _DYNAMIC_HELPERS:
        def read_avail(self) -> int:
            """
            Returns the number of bytes available to read in the input buffer starting from the
            current absolute stream position pointer of the API object.

            @return the number of bytes available to read
            """
            return self._run(self._aio.read_avail())

    if not _DYNAMIC_HELPERS:
        def queryLine(self, query: str, maxWait: int) -> str:
            """
            Sends a text line query to the serial port, and reads the reply, if any.
            This function is intended to be used when the serial port is configured for 'Line' protocol.

            @param query : the line query to send (without CR/LF)
            @param maxWait : the maximum number of milliseconds to wait for a reply.

            @return the next text line received after sending the text query, as a string.
                    Additional lines can be obtained by calling readLine or readMessages.

            On failure, throws an exception or returns an empty string.
            """
            return self._run(self._aio.queryLine(query, maxWait))

    if not _DYNAMIC_HELPERS:
        def queryHex(self, hexString: str, maxWait: int) -> str:
            """
            Sends a binary message to the serial port, and reads the reply, if any.
            This function is intended to be used when the serial port is configured for
            Frame-based protocol.

            @param hexString : the message to send, coded in hexadecimal
            @param maxWait : the maximum number of milliseconds to wait for a reply.

            @return the next frame received after sending the message, as a hex string.
                    Additional frames can be obtained by calling readHex or readMessages.

            On failure, throws an exception or returns an empty string.
            """
            return self._run(self._aio.queryHex(hexString, maxWait))

    if not _DYNAMIC_HELPERS:
        def uploadJob(self, jobfile: str, jsonDef: str) -> int:
            """
            Saves the job definition string (JSON data) into a job file.
            The job file can be later enabled using selectJob().

            @param jobfile : name of the job file to save on the device filesystem
            @param jsonDef : a string containing a JSON definition of the job

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.uploadJob(jobfile, jsonDef))

    if not _DYNAMIC_HELPERS:
        def selectJob(self, jobfile: str) -> int:
            """
            Load and start processing the specified job file. The file must have
            been previously created using the user interface or uploaded on the
            device filesystem using the uploadJob() function.

            @param jobfile : name of the job file (on the device filesystem)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.selectJob(jobfile))

    if not _DYNAMIC_HELPERS:
        def reset(self) -> int:
            """
            Clears the serial port buffer and resets counters to zero.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.reset())

    if not _DYNAMIC_HELPERS:
        def i2cSendBin(self, slaveAddr: int, buff: xarray) -> int:
            """
            Sends a one-way message (provided as a a binary buffer) to a device on the I2C bus.
            This function checks and reports communication errors on the I2C bus.

            @param slaveAddr : the 7-bit address of the slave device (without the direction bit)
            @param buff : the binary buffer to be sent

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.i2cSendBin(slaveAddr, buff))

    if not _DYNAMIC_HELPERS:
        def i2cSendArray(self, slaveAddr: int, values: list[int]) -> int:
            """
            Sends a one-way message (provided as a list of integer) to a device on the I2C bus.
            This function checks and reports communication errors on the I2C bus.

            @param slaveAddr : the 7-bit address of the slave device (without the direction bit)
            @param values : a list of data bytes to be sent

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.i2cSendArray(slaveAddr, values))

    if not _DYNAMIC_HELPERS:
        def i2cSendAndReceiveBin(self, slaveAddr: int, buff: xarray, rcvCount: int) -> xarray:
            """
            Sends a one-way message (provided as a a binary buffer) to a device on the I2C bus,
            then read back the specified number of bytes from device.
            This function checks and reports communication errors on the I2C bus.

            @param slaveAddr : the 7-bit address of the slave device (without the direction bit)
            @param buff : the binary buffer to be sent
            @param rcvCount : the number of bytes to receive once the data bytes are sent

            @return a list of bytes with the data received from slave device.

            On failure, throws an exception or returns an empty binary buffer.
            """
            return self._run(self._aio.i2cSendAndReceiveBin(slaveAddr, buff, rcvCount))

    if not _DYNAMIC_HELPERS:
        def i2cSendAndReceiveArray(self, slaveAddr: int, values: list[int], rcvCount: int) -> list[int]:
            """
            Sends a one-way message (provided as a list of integer) to a device on the I2C bus,
            then read back the specified number of bytes from device.
            This function checks and reports communication errors on the I2C bus.

            @param slaveAddr : the 7-bit address of the slave device (without the direction bit)
            @param values : a list of data bytes to be sent
            @param rcvCount : the number of bytes to receive once the data bytes are sent

            @return a list of bytes with the data received from slave device.

            On failure, throws an exception or returns an empty array.
            """
            return self._run(self._aio.i2cSendAndReceiveArray(slaveAddr, values, rcvCount))

    if not _DYNAMIC_HELPERS:
        def writeStr(self, codes: str) -> int:
            """
            Sends a text-encoded I2C code stream to the I2C bus, as is.
            An I2C code stream is a string made of hexadecimal data bytes,
            but that may also include the I2C state transitions code:
            "{S}" to emit a start condition,
            "{R}" for a repeated start condition,
            "{P}" for a stop condition,
            "xx" for receiving a data byte,
            "{A}" to ack a data byte received and
            "{N}" to nack a data byte received.
            If a newline ("\n") is included in the stream, the message
            will be terminated and a newline will also be added to the
            receive stream.

            @param codes : the code stream to send

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.writeStr(codes))

    if not _DYNAMIC_HELPERS:
        def writeLine(self, codes: str) -> int:
            """
            Sends a text-encoded I2C code stream to the I2C bus, and release the bus.
            An I2C code stream is a string made of hexadecimal data bytes,
            but that may also include the I2C state transitions code:
            "{S}" to emit a start condition,
            "{R}" for a repeated start condition,
            "{P}" for a stop condition,
            "xx" for receiving a data byte,
            "{A}" to ack a data byte received and
            "{N}" to nack a data byte received.
            At the end of the stream, a stop condition is added if missing
            and a newline is added to the receive buffer as well.

            @param codes : the code stream to send

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.writeLine(codes))

    if not _DYNAMIC_HELPERS:
        def writeByte(self, code: int) -> int:
            """
            Sends a single byte to the I2C bus. Depending on the I2C bus state, the byte
            will be interpreted as an address byte or a data byte.

            @param code : the byte to send

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.writeByte(code))

    if not _DYNAMIC_HELPERS:
        def writeHex(self, hexString: str) -> int:
            """
            Sends a byte sequence (provided as a hexadecimal string) to the I2C bus.
            Depending on the I2C bus state, the first byte will be interpreted as an
            address byte or a data byte.

            @param hexString : a string of hexadecimal byte codes

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.writeHex(hexString))

    if not _DYNAMIC_HELPERS:
        def writeBin(self, buff: xarray) -> int:
            """
            Sends a binary buffer to the I2C bus, as is.
            Depending on the I2C bus state, the first byte will be interpreted
            as an address byte or a data byte.

            @param buff : the binary buffer to send

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.writeBin(buff))

    if not _DYNAMIC_HELPERS:
        def writeArray(self, byteList: list[int]) -> int:
            """
            Sends a byte sequence (provided as a list of bytes) to the I2C bus.
            Depending on the I2C bus state, the first byte will be interpreted as an
            address byte or a data byte.

            @param byteList : a list of byte codes

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.writeArray(byteList))

    if not _DYNAMIC_HELPERS:
        def snoopMessagesEx(self, maxWait: int, maxMsg: int) -> list[YI2cSnoopingRecord]:
            """
            Retrieves messages (both direction) in the I2C port buffer, starting at current position.

            If no message is found, the search waits for one up to the specified maximum timeout
            (in milliseconds).

            @param maxWait : the maximum number of milliseconds to wait for a message if none is found
                    in the receive buffer.
            @param maxMsg : the maximum number of messages to be returned by the function; up to 254.

            @return an array of YI2cSnoopingRecord objects containing the messages found, if any.

            On failure, throws an exception or returns an empty array.
            """
            return self._run(self._aio.snoopMessagesEx(maxWait, maxMsg))

    if not _DYNAMIC_HELPERS:
        def snoopMessages(self, maxWait: int) -> list[YI2cSnoopingRecord]:
            """
            Retrieves messages (both direction) in the I2C port buffer, starting at current position.

            If no message is found, the search waits for one up to the specified maximum timeout
            (in milliseconds).

            @param maxWait : the maximum number of milliseconds to wait for a message if none is found
                    in the receive buffer.

            @return an array of YI2cSnoopingRecord objects containing the messages found, if any.

            On failure, throws an exception or returns an empty array.
            """
            return self._run(self._aio.snoopMessages(maxWait))

    # --- (end of generated code: YI2cPort implementation)




# ********************************************************************
#
#  $Id: helloworld.py 66200 2025-05-05 16:50:11Z seb $
#
#  An example that show how to use a  Yocto-Thermocouple
#
#  You can find more information on our web site:
#   Yocto-Thermocouple documentation:
#      https://www.yoctopuce.com/EN/products/yocto-thermocouple/doc.html
#   Python API Reference:
#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-python-EN.html
#
# *********************************************************************

import sys

from yoctolib.yocto_api import YRefParam, YAPI
from yoctolib.yocto_temperature import YTemperature


def die(msg):
    YAPI.FreeAPI()
    sys.exit(msg + ' (check USB cable)')


# the API use local USB devices through VirtualHub
errmsg = YRefParam()
if YAPI.RegisterHub("localhost", errmsg) != YAPI.SUCCESS:
    sys.exit("RegisterHub failed: " + errmsg.value)

# To use a specific device, invoke the script as
#   python helloworld.py [serial_number]
# or
#   python helloworld.py [logical_name]
target = 'any'
if len(sys.argv) > 1:
    target = sys.argv[1]

if target == 'any':
    # retrieve any temperature sensor
    sensor = YTemperature.FirstTemperature()
    if sensor is None:
        die('No module connected')
    target = sensor.get_serialNumber()

# retrieve specified functions
channel1 = YTemperature.FindTemperature(target + '.temperature1')
channel2 = YTemperature.FindTemperature(target + '.temperature2')
if not channel1.isOnline():
    die("Yocto-Meteo '%s' not connected" % target)

print("Use device %s" % channel1.get_serialNumber())
while channel1.isOnline():
    print("Channel 1 temperature: %2.1f" % channel1.get_currentValue())
    print("Channel 2 temperature: %2.1f" % channel2.get_currentValue())
    print("  (press Ctrl-C to exit)")
    YAPI.Sleep(1000)
YAPI.FreeAPI()



# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YDigitalIO
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YDigitalIO
version: PATCH_WITH_VERSION
requires: yocto_digitalio_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_digitalio_aio import YDigitalIO as YDigitalIO_aio
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (YDigitalIO class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YDigitalIOValueCallback = Union[Callable[['YDigitalIO', str], Awaitable[None]], None]
    except TypeError:
        YDigitalIOValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YDigitalIO(YFunction):
    """
    The YDigitalIO class allows you drive a Yoctopuce digital input/output port.
    It can be used to set up the direction of each channel, to read the state of each channel
    and to switch the state of each channel configures as an output.
    You can work on all channels at once, or one by one. Most functions
    use a binary representation for channels where bit 0 matches channel #0 , bit 1 matches channel
    #1 and so on. If you are not familiar with numbers binary representation, you will find more
    information here: https://en.wikipedia.org/wiki/Binary_number#Representation. It is also possible
    to automatically generate short pulses of a determined duration. Electrical behavior
    of each I/O can be modified (open drain and reverse polarity).

    """
    _aio: YDigitalIO_aio
    # --- (end of YDigitalIO class start)
    if not _IS_MICROPYTHON:
        # --- (YDigitalIO return codes)
        PORTSTATE_INVALID: Final[int] = YAPI.INVALID_UINT
        PORTDIRECTION_INVALID: Final[int] = YAPI.INVALID_UINT
        PORTOPENDRAIN_INVALID: Final[int] = YAPI.INVALID_UINT
        PORTPOLARITY_INVALID: Final[int] = YAPI.INVALID_UINT
        PORTDIAGS_INVALID: Final[int] = YAPI.INVALID_UINT
        PORTSIZE_INVALID: Final[int] = YAPI.INVALID_UINT
        COMMAND_INVALID: Final[str] = YAPI.INVALID_STRING
        OUTPUTVOLTAGE_USB_5V: Final[int] = 0
        OUTPUTVOLTAGE_USB_3V: Final[int] = 1
        OUTPUTVOLTAGE_EXT_V: Final[int] = 2
        OUTPUTVOLTAGE_INVALID: Final[int] = -1
        # --- (end of YDigitalIO return codes)


    # --- (YDigitalIO implementation)

    @classmethod
    def FirstDigitalIO(cls) -> Union[YDigitalIO, None]:
        """
        Starts the enumeration of digital IO ports currently accessible.
        Use the method YDigitalIO.nextDigitalIO() to iterate on
        next digital IO ports.

        @return a pointer to a YDigitalIO object, corresponding to
                the first digital IO port currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YDigitalIO_aio.FirstDigitalIO())

    @classmethod
    def FirstDigitalIOInContext(cls, yctx: YAPIContext) -> Union[YDigitalIO, None]:
        """
        Starts the enumeration of digital IO ports currently accessible.
        Use the method YDigitalIO.nextDigitalIO() to iterate on
        next digital IO ports.

        @param yctx : a YAPI context.

        @return a pointer to a YDigitalIO object, corresponding to
                the first digital IO port currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YDigitalIO_aio.FirstDigitalIOInContext(yctx))

    def nextDigitalIO(self):
        """
        Continues the enumeration of digital IO ports started using yFirstDigitalIO().
        Caution: You can't make any assumption about the returned digital IO ports order.
        If you want to find a specific a digital IO port, use DigitalIO.findDigitalIO()
        and a hardwareID or a logical name.

        @return a pointer to a YDigitalIO object, corresponding to
                a digital IO port currently online, or a None pointer
                if there are no more digital IO ports to enumerate.
        """
        return self._proxy(type(self), self._aio.nextDigitalIO())

    if not _DYNAMIC_HELPERS:
        def get_portState(self) -> int:
            """
            Returns the digital IO port state as an integer with each bit
            representing a channel.
            value 0 = 0b00000000 -> all channels are OFF
            value 1 = 0b00000001 -> channel #0 is ON
            value 2 = 0b00000010 -> channel #1 is ON
            value 3 = 0b00000011 -> channels #0 and #1 are ON
            value 4 = 0b00000100 -> channel #2 is ON
            and so on...

            @return an integer corresponding to the digital IO port state as an integer with each bit
                    representing a channel

            On failure, throws an exception or returns YDigitalIO.PORTSTATE_INVALID.
            """
            return self._run(self._aio.get_portState())

    if not _DYNAMIC_HELPERS:
        def set_portState(self, newval: int) -> int:
            """
            Changes the state of all digital IO port's channels at once: the parameter
            is an integer where each bit represents a channel, with bit 0 matching channel #0.
            To set all channels to  0 -> 0b00000000 -> parameter = 0
            To set channel #0 to 1 -> 0b00000001 -> parameter =  1
            To set channel #1 to  1 -> 0b00000010 -> parameter = 2
            To set channel #0 and #1 -> 0b00000011 -> parameter =  3
            To set channel #2 to 1 -> 0b00000100 -> parameter =  4
            an so on....
            Only channels configured as outputs will be affecter, according to the value
            configured using set_portDirection.

            @param newval : an integer corresponding to the state of all digital IO port's channels at once: the parameter
                    is an integer where each bit represents a channel, with bit 0 matching channel #0

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_portState(newval))

    if not _DYNAMIC_HELPERS:
        def get_portDirection(self) -> int:
            """
            Returns the I/O direction of all channels of the port (bitmap): 0 makes a bit an input, 1 makes it an output.

            @return an integer corresponding to the I/O direction of all channels of the port (bitmap): 0 makes
            a bit an input, 1 makes it an output

            On failure, throws an exception or returns YDigitalIO.PORTDIRECTION_INVALID.
            """
            return self._run(self._aio.get_portDirection())

    if not _DYNAMIC_HELPERS:
        def set_portDirection(self, newval: int) -> int:
            """
            Changes the I/O direction of all channels of the port (bitmap): 0 makes a bit an input, 1 makes it an output.
            Remember to call the saveToFlash() method  to make sure the setting is kept after a reboot.

            @param newval : an integer corresponding to the I/O direction of all channels of the port (bitmap):
            0 makes a bit an input, 1 makes it an output

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_portDirection(newval))

    if not _DYNAMIC_HELPERS:
        def get_portOpenDrain(self) -> int:
            """
            Returns the electrical interface for each bit of the port. For each bit set to 0  the matching I/O
            works in the regular,
            intuitive way, for each bit set to 1, the I/O works in reverse mode.

            @return an integer corresponding to the electrical interface for each bit of the port

            On failure, throws an exception or returns YDigitalIO.PORTOPENDRAIN_INVALID.
            """
            return self._run(self._aio.get_portOpenDrain())

    if not _DYNAMIC_HELPERS:
        def set_portOpenDrain(self, newval: int) -> int:
            """
            Changes the electrical interface for each bit of the port. 0 makes a bit a regular input/output, 1 makes
            it an open-drain (open-collector) input/output. Remember to call the
            saveToFlash() method  to make sure the setting is kept after a reboot.

            @param newval : an integer corresponding to the electrical interface for each bit of the port

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_portOpenDrain(newval))

    if not _DYNAMIC_HELPERS:
        def get_portPolarity(self) -> int:
            """
            Returns the polarity of all the bits of the port.  For each bit set to 0, the matching I/O works the regular,
            intuitive way; for each bit set to 1, the I/O works in reverse mode.

            @return an integer corresponding to the polarity of all the bits of the port

            On failure, throws an exception or returns YDigitalIO.PORTPOLARITY_INVALID.
            """
            return self._run(self._aio.get_portPolarity())

    if not _DYNAMIC_HELPERS:
        def set_portPolarity(self, newval: int) -> int:
            """
            Changes the polarity of all the bits of the port: For each bit set to 0, the matching I/O works the regular,
            intuitive way; for each bit set to 1, the I/O works in reverse mode.
            Remember to call the saveToFlash() method  to make sure the setting will be kept after a reboot.

            @param newval : an integer corresponding to the polarity of all the bits of the port: For each bit
            set to 0, the matching I/O works the regular,
                    intuitive way; for each bit set to 1, the I/O works in reverse mode

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_portPolarity(newval))

    if not _DYNAMIC_HELPERS:
        def get_portDiags(self) -> int:
            """
            Returns the port state diagnostics. Bit 0 indicates a shortcut on output 0, etc.
            Bit 8 indicates a power failure, and bit 9 signals overheating (overcurrent).
            During normal use, all diagnostic bits should stay clear.

            @return an integer corresponding to the port state diagnostics

            On failure, throws an exception or returns YDigitalIO.PORTDIAGS_INVALID.
            """
            return self._run(self._aio.get_portDiags())

    if not _DYNAMIC_HELPERS:
        def get_portSize(self) -> int:
            """
            Returns the number of bits (i.e. channels)implemented in the I/O port.

            @return an integer corresponding to the number of bits (i.e

            On failure, throws an exception or returns YDigitalIO.PORTSIZE_INVALID.
            """
            return self._run(self._aio.get_portSize())

    if not _DYNAMIC_HELPERS:
        def get_outputVoltage(self) -> int:
            """
            Returns the voltage source used to drive output bits.

            @return a value among YDigitalIO.OUTPUTVOLTAGE_USB_5V, YDigitalIO.OUTPUTVOLTAGE_USB_3V and
            YDigitalIO.OUTPUTVOLTAGE_EXT_V corresponding to the voltage source used to drive output bits

            On failure, throws an exception or returns YDigitalIO.OUTPUTVOLTAGE_INVALID.
            """
            return self._run(self._aio.get_outputVoltage())

    if not _DYNAMIC_HELPERS:
        def set_outputVoltage(self, newval: int) -> int:
            """
            Changes the voltage source used to drive output bits.
            Remember to call the saveToFlash() method  to make sure the setting is kept after a reboot.

            @param newval : a value among YDigitalIO.OUTPUTVOLTAGE_USB_5V, YDigitalIO.OUTPUTVOLTAGE_USB_3V and
            YDigitalIO.OUTPUTVOLTAGE_EXT_V corresponding to the voltage source used to drive output bits

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_outputVoltage(newval))

    if not _DYNAMIC_HELPERS:
        def set_command(self, newval: str) -> int:
            return self._run(self._aio.set_command(newval))

    @classmethod
    def FindDigitalIO(cls, func: str) -> YDigitalIO:
        """
        Retrieves a digital IO port for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the digital IO port is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YDigitalIO.isOnline() to test if the digital IO port is
        indeed online at a given time. In case of ambiguity when looking for
        a digital IO port by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the digital IO port, for instance
                YMINIIO0.digitalIO.

        @return a YDigitalIO object allowing you to drive the digital IO port.
        """
        return cls._proxy(cls, YDigitalIO_aio.FindDigitalIO(func))

    @classmethod
    def FindDigitalIOInContext(cls, yctx: YAPIContext, func: str) -> YDigitalIO:
        """
        Retrieves a digital IO port for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the digital IO port is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YDigitalIO.isOnline() to test if the digital IO port is
        indeed online at a given time. In case of ambiguity when looking for
        a digital IO port by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the digital IO port, for instance
                YMINIIO0.digitalIO.

        @return a YDigitalIO object allowing you to drive the digital IO port.
        """
        return cls._proxy(cls, YDigitalIO_aio.FindDigitalIOInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YDigitalIOValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _DYNAMIC_HELPERS:
        def set_bitState(self, bitno: int, bitstate: int) -> int:
            """
            Sets a single bit (i.e. channel) of the I/O port.

            @param bitno : the bit number; lowest bit has index 0
            @param bitstate : the state of the bit (1 or 0)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_bitState(bitno, bitstate))

    if not _DYNAMIC_HELPERS:
        def get_bitState(self, bitno: int) -> int:
            """
            Returns the state of a single bit (i.e. channel)  of the I/O port.

            @param bitno : the bit number; lowest bit has index 0

            @return the bit state (0 or 1)

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.get_bitState(bitno))

    if not _DYNAMIC_HELPERS:
        def toggle_bitState(self, bitno: int) -> int:
            """
            Reverts a single bit (i.e. channel) of the I/O port.

            @param bitno : the bit number; lowest bit has index 0

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.toggle_bitState(bitno))

    if not _DYNAMIC_HELPERS:
        def set_bitDirection(self, bitno: int, bitdirection: int) -> int:
            """
            Changes  the direction of a single bit (i.e. channel) from the I/O port.

            @param bitno : the bit number; lowest bit has index 0
            @param bitdirection : direction to set, 0 makes the bit an input, 1 makes it an output.
                    Remember to call the   saveToFlash() method to make sure the setting is kept after a reboot.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_bitDirection(bitno, bitdirection))

    if not _DYNAMIC_HELPERS:
        def get_bitDirection(self, bitno: int) -> int:
            """
            Returns the direction of a single bit (i.e. channel) from the I/O port (0 means the bit is an
            input, 1  an output).

            @param bitno : the bit number; lowest bit has index 0

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.get_bitDirection(bitno))

    if not _DYNAMIC_HELPERS:
        def set_bitPolarity(self, bitno: int, bitpolarity: int) -> int:
            """
            Changes the polarity of a single bit from the I/O port.

            @param bitno : the bit number; lowest bit has index 0.
            @param bitpolarity : polarity to set, 0 makes the I/O work in regular mode, 1 makes the I/O  works
            in reverse mode.
                    Remember to call the   saveToFlash() method to make sure the setting is kept after a reboot.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_bitPolarity(bitno, bitpolarity))

    if not _DYNAMIC_HELPERS:
        def get_bitPolarity(self, bitno: int) -> int:
            """
            Returns the polarity of a single bit from the I/O port (0 means the I/O works in regular mode, 1
            means the I/O  works in reverse mode).

            @param bitno : the bit number; lowest bit has index 0

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.get_bitPolarity(bitno))

    if not _DYNAMIC_HELPERS:
        def set_bitOpenDrain(self, bitno: int, opendrain: int) -> int:
            """
            Changes  the electrical interface of a single bit from the I/O port.

            @param bitno : the bit number; lowest bit has index 0
            @param opendrain : 0 makes a bit a regular input/output, 1 makes
                    it an open-drain (open-collector) input/output. Remember to call the
                    saveToFlash() method to make sure the setting is kept after a reboot.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_bitOpenDrain(bitno, opendrain))

    if not _DYNAMIC_HELPERS:
        def get_bitOpenDrain(self, bitno: int) -> int:
            """
            Returns the type of electrical interface of a single bit from the I/O port. (0 means the bit is an
            input, 1  an output).

            @param bitno : the bit number; lowest bit has index 0

            @return   0 means the a bit is a regular input/output, 1 means the bit is an open-drain
                    (open-collector) input/output.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.get_bitOpenDrain(bitno))

    if not _DYNAMIC_HELPERS:
        def pulse(self, bitno: int, ms_duration: int) -> int:
            """
            Triggers a pulse on a single bit for a specified duration. The specified bit
            will be turned to 1, and then back to 0 after the given duration.

            @param bitno : the bit number; lowest bit has index 0
            @param ms_duration : desired pulse duration in milliseconds. Be aware that the device time
                    resolution is not guaranteed up to the millisecond.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.pulse(bitno, ms_duration))

    if not _DYNAMIC_HELPERS:
        def delayedPulse(self, bitno: int, ms_delay: int, ms_duration: int) -> int:
            """
            Schedules a pulse on a single bit for a specified duration. The specified bit
            will be turned to 1, and then back to 0 after the given duration.

            @param bitno : the bit number; lowest bit has index 0
            @param ms_delay : waiting time before the pulse, in milliseconds
            @param ms_duration : desired pulse duration in milliseconds. Be aware that the device time
                    resolution is not guaranteed up to the millisecond.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.delayedPulse(bitno, ms_delay, ms_duration))

    # --- (end of YDigitalIO implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: yocto_gyro.py 66072 2025-04-30 06:59:12Z mvuilleu $
#
#  Implements the asyncio YGyro API for Gyro functions
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YGyro
version: PATCH_WITH_VERSION
requires: yocto_api
requires: yocto_display_aio
"""
from __future__ import annotations
import sys, math

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import const, _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True
    _DYNAMIC_HELPERS: Final[bool] = True

from .yocto_display_aio import YGyro as YGyro_aio
from .yocto_api import (
    YAPIContext, YAPI, YSensor, YMeasure
)

# --- (generated code: YGyro class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YGyroValueCallback = Union[Callable[['YGyro', str], Awaitable[None]], None]
        YGyroTimedReportCallback = Union[Callable[['YGyro', YMeasure], Awaitable[None]], None]
    except TypeError:
        YGyroValueCallback = Union[Callable, Awaitable]
        YGyroTimedReportCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YGyro(YSensor):
    """
    The YGyro class allows you to read and configure Yoctopuce gyroscopes.
    It inherits from YSensor class the core functions to read measurements,
    to register callback functions, and to access the autonomous datalogger.
    This class adds the possibility to access x, y and z components of the rotation
    vector separately, as well as the possibility to deal with quaternion-based
    orientation estimates.

    """
    _aio: YGyro_aio
    # --- (end of generated code: YGyro class start)
    if not _IS_MICROPYTHON:
        # --- (generated code: YGyro return codes)
        BANDWIDTH_INVALID: Final[int] = YAPI.INVALID_UINT
        XVALUE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        YVALUE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        ZVALUE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        # --- (end of generated code: YGyro return codes)

    # --- (generated code: YGyro attributes declaration)
    _bandwidth: int
    _xValue: float
    _yValue: float
    _zValue: float
    _valueCallbackGyro: YGyroValueCallback
    _timedReportCallbackGyro: YGyroTimedReportCallback
    _qt_stamp: int
    _qt_w: YQt
    _qt_x: YQt
    _qt_y: YQt
    _qt_z: YQt
    _w: float
    _x: float
    _y: float
    _z: float
    _angles_stamp: int
    _head: float
    _pitch: float
    _roll: float
    _quatCallback: YQuatCallback
    _anglesCallback: YAnglesCallback
    # --- (end of generated code: YGyro attributes declaration)

    # --- (generated code: YGyro implementation)

    @classmethod
    def FirstGyro(cls) -> Union[YGyro, None]:
        """
        Starts the enumeration of gyroscopes currently accessible.
        Use the method YGyro.nextGyro() to iterate on
        next gyroscopes.

        @return a pointer to a YGyro object, corresponding to
                the first gyro currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YGyro_aio.FirstGyro())

    @classmethod
    def FirstGyroInContext(cls, yctx: YAPIContext) -> Union[YGyro, None]:
        """
        Starts the enumeration of gyroscopes currently accessible.
        Use the method YGyro.nextGyro() to iterate on
        next gyroscopes.

        @param yctx : a YAPI context.

        @return a pointer to a YGyro object, corresponding to
                the first gyro currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YGyro_aio.FirstGyroInContext(yctx))

    def nextGyro(self):
        """
        Continues the enumeration of gyroscopes started using yFirstGyro().
        Caution: You can't make any assumption about the returned gyroscopes order.
        If you want to find a specific a gyroscope, use Gyro.findGyro()
        and a hardwareID or a logical name.

        @return a pointer to a YGyro object, corresponding to
                a gyroscope currently online, or a None pointer
                if there are no more gyroscopes to enumerate.
        """
        return self._proxy(type(self), self._aio.nextGyro())

    if not _DYNAMIC_HELPERS:
        def get_bandwidth(self) -> int:
            """
            Returns the measure update frequency, measured in Hz.

            @return an integer corresponding to the measure update frequency, measured in Hz

            On failure, throws an exception or returns YGyro.BANDWIDTH_INVALID.
            """
            return self._run(self._aio.get_bandwidth())

    if not _DYNAMIC_HELPERS:
        def set_bandwidth(self, newval: int) -> int:
            """
            Changes the measure update frequency, measured in Hz. When the
            frequency is lower, the device performs averaging.
            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : an integer corresponding to the measure update frequency, measured in Hz

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_bandwidth(newval))

    if not _DYNAMIC_HELPERS:
        def get_xValue(self) -> float:
            """
            Returns the angular velocity around the X axis of the device, as a floating point number.

            @return a floating point number corresponding to the angular velocity around the X axis of the
            device, as a floating point number

            On failure, throws an exception or returns YGyro.XVALUE_INVALID.
            """
            return self._run(self._aio.get_xValue())

    if not _DYNAMIC_HELPERS:
        def get_yValue(self) -> float:
            """
            Returns the angular velocity around the Y axis of the device, as a floating point number.

            @return a floating point number corresponding to the angular velocity around the Y axis of the
            device, as a floating point number

            On failure, throws an exception or returns YGyro.YVALUE_INVALID.
            """
            return self._run(self._aio.get_yValue())

    if not _DYNAMIC_HELPERS:
        def get_zValue(self) -> float:
            """
            Returns the angular velocity around the Z axis of the device, as a floating point number.

            @return a floating point number corresponding to the angular velocity around the Z axis of the
            device, as a floating point number

            On failure, throws an exception or returns YGyro.ZVALUE_INVALID.
            """
            return self._run(self._aio.get_zValue())

    @classmethod
    def FindGyro(cls, func: str) -> YGyro:
        """
        Retrieves a gyroscope for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the gyroscope is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YGyro.isOnline() to test if the gyroscope is
        indeed online at a given time. In case of ambiguity when looking for
        a gyroscope by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the gyroscope, for instance
                Y3DMK002.gyro.

        @return a YGyro object allowing you to drive the gyroscope.
        """
        return cls._proxy(cls, YGyro_aio.FindGyro(func))

    @classmethod
    def FindGyroInContext(cls, yctx: YAPIContext, func: str) -> YGyro:
        """
        Retrieves a gyroscope for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the gyroscope is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YGyro.isOnline() to test if the gyroscope is
        indeed online at a given time. In case of ambiguity when looking for
        a gyroscope by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the gyroscope, for instance
                Y3DMK002.gyro.

        @return a YGyro object allowing you to drive the gyroscope.
        """
        return cls._proxy(cls, YGyro_aio.FindGyroInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YGyroValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _IS_MICROPYTHON:
        def registerTimedReportCallback(self, callback: YGyroTimedReportCallback) -> int:
            """
            Registers the callback function that is invoked on every periodic timed notification.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and an YMeasure object describing
                    the new advertised value.
            @noreturn
            """
            return super().registerTimedReportCallback(callback)

    if not _DYNAMIC_HELPERS:
        def get_roll(self) -> float:
            """
            Returns the estimated roll angle, based on the integration of
            gyroscopic measures combined with acceleration and
            magnetic field measurements.
            The axis corresponding to the roll angle can be mapped to any
            of the device X, Y or Z physical directions using methods of
            the class YRefFrame.

            @return a floating-point number corresponding to roll angle
                    in degrees, between -180 and +180.
            """
            return self._run(self._aio.get_roll())

    if not _DYNAMIC_HELPERS:
        def get_pitch(self) -> float:
            """
            Returns the estimated pitch angle, based on the integration of
            gyroscopic measures combined with acceleration and
            magnetic field measurements.
            The axis corresponding to the pitch angle can be mapped to any
            of the device X, Y or Z physical directions using methods of
            the class YRefFrame.

            @return a floating-point number corresponding to pitch angle
                    in degrees, between -90 and +90.
            """
            return self._run(self._aio.get_pitch())

    if not _DYNAMIC_HELPERS:
        def get_heading(self) -> float:
            """
            Returns the estimated heading angle, based on the integration of
            gyroscopic measures combined with acceleration and
            magnetic field measurements.
            The axis corresponding to the heading can be mapped to any
            of the device X, Y or Z physical directions using methods of
            the class YRefFrame.

            @return a floating-point number corresponding to heading
                    in degrees, between 0 and 360.
            """
            return self._run(self._aio.get_heading())

    if not _DYNAMIC_HELPERS:
        def get_quaternionW(self) -> float:
            """
            Returns the w component (real part) of the quaternion
            describing the device estimated orientation, based on the
            integration of gyroscopic measures combined with acceleration and
            magnetic field measurements.

            @return a floating-point number corresponding to the w
                    component of the quaternion.
            """
            return self._run(self._aio.get_quaternionW())

    if not _DYNAMIC_HELPERS:
        def get_quaternionX(self) -> float:
            """
            Returns the x component of the quaternion
            describing the device estimated orientation, based on the
            integration of gyroscopic measures combined with acceleration and
            magnetic field measurements. The x component is
            mostly correlated with rotations on the roll axis.

            @return a floating-point number corresponding to the x
                    component of the quaternion.
            """
            return self._run(self._aio.get_quaternionX())

    if not _DYNAMIC_HELPERS:
        def get_quaternionY(self) -> float:
            """
            Returns the y component of the quaternion
            describing the device estimated orientation, based on the
            integration of gyroscopic measures combined with acceleration and
            magnetic field measurements. The y component is
            mostly correlated with rotations on the pitch axis.

            @return a floating-point number corresponding to the y
                    component of the quaternion.
            """
            return self._run(self._aio.get_quaternionY())

    if not _DYNAMIC_HELPERS:
        def get_quaternionZ(self) -> float:
            """
            Returns the x component of the quaternion
            describing the device estimated orientation, based on the
            integration of gyroscopic measures combined with acceleration and
            magnetic field measurements. The x component is
            mostly correlated with changes of heading.

            @return a floating-point number corresponding to the z
                    component of the quaternion.
            """
            return self._run(self._aio.get_quaternionZ())

    def registerQuaternionCallback(self, callback: YQuatCallback) -> int:
        """
        Registers a callback function that will be invoked each time that the estimated
        device orientation has changed. The call frequency is typically around 95Hz during a move.
        The callback is invoked only during the execution of ySleep or yHandleEvents.
        This provides control over the time when the callback is triggered.
        For good responsiveness, remember to call one of these two functions periodically.
        To unregister a callback, pass a None pointer as argument.

        @param callback : the callback function to invoke, or a None pointer.
                The callback function should take five arguments:
                the YGyro object of the turning device, and the floating
                point values of the four components w, x, y and z
                (as floating-point numbers).
        @noreturn
        """
        return self._run(self._aio.registerQuaternionCallback(self._proxyCb(type(self), callback)))

    def registerAnglesCallback(self, callback: YAnglesCallback) -> int:
        """
        Registers a callback function that will be invoked each time that the estimated
        device orientation has changed. The call frequency is typically around 95Hz during a move.
        The callback is invoked only during the execution of ySleep or yHandleEvents.
        This provides control over the time when the callback is triggered.
        For good responsiveness, remember to call one of these two functions periodically.
        To unregister a callback, pass a None pointer as argument.

        @param callback : the callback function to invoke, or a None pointer.
                The callback function should take four arguments:
                the YGyro object of the turning device, and the floating
                point values of the three angles roll, pitch and heading
                in degrees (as floating-point numbers).
        @noreturn
        """
        return self._run(self._aio.registerAnglesCallback(self._proxyCb(type(self), callback)))

    # --- (end of generated code: YGyro implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YProximity
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YProximity
version: PATCH_WITH_VERSION
requires: yocto_proximity_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_proximity_aio import YProximity as YProximity_aio
from .yocto_api import (
    YAPIContext, YAPI, YSensor, YMeasure
)

# --- (YProximity class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YProximityValueCallback = Union[Callable[['YProximity', str], Awaitable[None]], None]
        YProximityTimedReportCallback = Union[Callable[['YProximity', YMeasure], Awaitable[None]], None]
    except TypeError:
        YProximityValueCallback = Union[Callable, Awaitable]
        YProximityTimedReportCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YProximity(YSensor):
    """
    The YProximity class allows you to read and configure Yoctopuce proximity sensors.
    It inherits from YSensor class the core functions to read measures,
    to register callback functions, and to access the autonomous datalogger.
    This class adds the ability to set up a detection threshold and to count the
    number of detected state changes.

    """
    _aio: YProximity_aio
    # --- (end of YProximity class start)
    if not _IS_MICROPYTHON:
        # --- (YProximity return codes)
        SIGNALVALUE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        DETECTIONTHRESHOLD_INVALID: Final[int] = YAPI.INVALID_UINT
        DETECTIONHYSTERESIS_INVALID: Final[int] = YAPI.INVALID_UINT
        PRESENCEMINTIME_INVALID: Final[int] = YAPI.INVALID_UINT
        REMOVALMINTIME_INVALID: Final[int] = YAPI.INVALID_UINT
        LASTTIMEAPPROACHED_INVALID: Final[int] = YAPI.INVALID_LONG
        LASTTIMEREMOVED_INVALID: Final[int] = YAPI.INVALID_LONG
        PULSECOUNTER_INVALID: Final[int] = YAPI.INVALID_LONG
        PULSETIMER_INVALID: Final[int] = YAPI.INVALID_LONG
        ISPRESENT_FALSE: Final[int] = 0
        ISPRESENT_TRUE: Final[int] = 1
        ISPRESENT_INVALID: Final[int] = -1
        PROXIMITYREPORTMODE_NUMERIC: Final[int] = 0
        PROXIMITYREPORTMODE_PRESENCE: Final[int] = 1
        PROXIMITYREPORTMODE_PULSECOUNT: Final[int] = 2
        PROXIMITYREPORTMODE_INVALID: Final[int] = -1
        # --- (end of YProximity return codes)


    # --- (YProximity implementation)

    @classmethod
    def FirstProximity(cls) -> Union[YProximity, None]:
        """
        Starts the enumeration of proximity sensors currently accessible.
        Use the method YProximity.nextProximity() to iterate on
        next proximity sensors.

        @return a pointer to a YProximity object, corresponding to
                the first proximity sensor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YProximity_aio.FirstProximity())

    @classmethod
    def FirstProximityInContext(cls, yctx: YAPIContext) -> Union[YProximity, None]:
        """
        Starts the enumeration of proximity sensors currently accessible.
        Use the method YProximity.nextProximity() to iterate on
        next proximity sensors.

        @param yctx : a YAPI context.

        @return a pointer to a YProximity object, corresponding to
                the first proximity sensor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YProximity_aio.FirstProximityInContext(yctx))

    def nextProximity(self):
        """
        Continues the enumeration of proximity sensors started using yFirstProximity().
        Caution: You can't make any assumption about the returned proximity sensors order.
        If you want to find a specific a proximity sensor, use Proximity.findProximity()
        and a hardwareID or a logical name.

        @return a pointer to a YProximity object, corresponding to
                a proximity sensor currently online, or a None pointer
                if there are no more proximity sensors to enumerate.
        """
        return self._proxy(type(self), self._aio.nextProximity())

    if not _DYNAMIC_HELPERS:
        def get_signalValue(self) -> float:
            """
            Returns the current value of signal measured by the proximity sensor.

            @return a floating point number corresponding to the current value of signal measured by the proximity sensor

            On failure, throws an exception or returns YProximity.SIGNALVALUE_INVALID.
            """
            return self._run(self._aio.get_signalValue())

    if not _DYNAMIC_HELPERS:
        def get_detectionThreshold(self) -> int:
            """
            Returns the threshold used to determine the logical state of the proximity sensor, when considered
            as a binary input (on/off).

            @return an integer corresponding to the threshold used to determine the logical state of the
            proximity sensor, when considered
                    as a binary input (on/off)

            On failure, throws an exception or returns YProximity.DETECTIONTHRESHOLD_INVALID.
            """
            return self._run(self._aio.get_detectionThreshold())

    if not _DYNAMIC_HELPERS:
        def set_detectionThreshold(self, newval: int) -> int:
            """
            Changes the threshold used to determine the logical state of the proximity sensor, when considered
            as a binary input (on/off).
            Remember to call the saveToFlash() method of the module if the modification must be kept.

            @param newval : an integer corresponding to the threshold used to determine the logical state of
            the proximity sensor, when considered
                    as a binary input (on/off)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_detectionThreshold(newval))

    if not _DYNAMIC_HELPERS:
        def get_detectionHysteresis(self) -> int:
            """
            Returns the hysteresis used to determine the logical state of the proximity sensor, when considered
            as a binary input (on/off).

            @return an integer corresponding to the hysteresis used to determine the logical state of the
            proximity sensor, when considered
                    as a binary input (on/off)

            On failure, throws an exception or returns YProximity.DETECTIONHYSTERESIS_INVALID.
            """
            return self._run(self._aio.get_detectionHysteresis())

    if not _DYNAMIC_HELPERS:
        def set_detectionHysteresis(self, newval: int) -> int:
            """
            Changes the hysteresis used to determine the logical state of the proximity sensor, when considered
            as a binary input (on/off).
            Remember to call the saveToFlash() method of the module if the modification must be kept.

            @param newval : an integer corresponding to the hysteresis used to determine the logical state of
            the proximity sensor, when considered
                    as a binary input (on/off)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_detectionHysteresis(newval))

    if not _DYNAMIC_HELPERS:
        def get_presenceMinTime(self) -> int:
            """
            Returns the minimal detection duration before signalling a presence event. Any shorter detection is
            considered as noise or bounce (false positive) and filtered out.

            @return an integer corresponding to the minimal detection duration before signalling a presence event

            On failure, throws an exception or returns YProximity.PRESENCEMINTIME_INVALID.
            """
            return self._run(self._aio.get_presenceMinTime())

    if not _DYNAMIC_HELPERS:
        def set_presenceMinTime(self, newval: int) -> int:
            """
            Changes the minimal detection duration before signalling a presence event. Any shorter detection is
            considered as noise or bounce (false positive) and filtered out.
            Remember to call the saveToFlash() method of the module if the modification must be kept.

            @param newval : an integer corresponding to the minimal detection duration before signalling a presence event

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_presenceMinTime(newval))

    if not _DYNAMIC_HELPERS:
        def get_removalMinTime(self) -> int:
            """
            Returns the minimal detection duration before signalling a removal event. Any shorter detection is
            considered as noise or bounce (false positive) and filtered out.

            @return an integer corresponding to the minimal detection duration before signalling a removal event

            On failure, throws an exception or returns YProximity.REMOVALMINTIME_INVALID.
            """
            return self._run(self._aio.get_removalMinTime())

    if not _DYNAMIC_HELPERS:
        def set_removalMinTime(self, newval: int) -> int:
            """
            Changes the minimal detection duration before signalling a removal event. Any shorter detection is
            considered as noise or bounce (false positive) and filtered out.
            Remember to call the saveToFlash() method of the module if the modification must be kept.

            @param newval : an integer corresponding to the minimal detection duration before signalling a removal event

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_removalMinTime(newval))

    if not _DYNAMIC_HELPERS:
        def get_isPresent(self) -> int:
            """
            Returns true if the input (considered as binary) is active (detection value is smaller than the
            specified threshold), and false otherwise.

            @return either YProximity.ISPRESENT_FALSE or YProximity.ISPRESENT_TRUE, according to true if the
            input (considered as binary) is active (detection value is smaller than the specified threshold),
            and false otherwise

            On failure, throws an exception or returns YProximity.ISPRESENT_INVALID.
            """
            return self._run(self._aio.get_isPresent())

    if not _DYNAMIC_HELPERS:
        def get_lastTimeApproached(self) -> int:
            """
            Returns the number of elapsed milliseconds between the module power on and the last observed
            detection (the input contact transitioned from absent to present).

            @return an integer corresponding to the number of elapsed milliseconds between the module power on
            and the last observed
                    detection (the input contact transitioned from absent to present)

            On failure, throws an exception or returns YProximity.LASTTIMEAPPROACHED_INVALID.
            """
            return self._run(self._aio.get_lastTimeApproached())

    if not _DYNAMIC_HELPERS:
        def get_lastTimeRemoved(self) -> int:
            """
            Returns the number of elapsed milliseconds between the module power on and the last observed
            detection (the input contact transitioned from present to absent).

            @return an integer corresponding to the number of elapsed milliseconds between the module power on
            and the last observed
                    detection (the input contact transitioned from present to absent)

            On failure, throws an exception or returns YProximity.LASTTIMEREMOVED_INVALID.
            """
            return self._run(self._aio.get_lastTimeRemoved())

    if not _DYNAMIC_HELPERS:
        def get_pulseCounter(self) -> int:
            """
            Returns the pulse counter value. The value is a 32 bit integer. In case
            of overflow (>=2^32), the counter will wrap. To reset the counter, just
            call the resetCounter() method.

            @return an integer corresponding to the pulse counter value

            On failure, throws an exception or returns YProximity.PULSECOUNTER_INVALID.
            """
            return self._run(self._aio.get_pulseCounter())

    if not _DYNAMIC_HELPERS:
        def set_pulseCounter(self, newval: int) -> int:
            return self._run(self._aio.set_pulseCounter(newval))

    if not _DYNAMIC_HELPERS:
        def get_pulseTimer(self) -> int:
            """
            Returns the timer of the pulse counter (ms).

            @return an integer corresponding to the timer of the pulse counter (ms)

            On failure, throws an exception or returns YProximity.PULSETIMER_INVALID.
            """
            return self._run(self._aio.get_pulseTimer())

    if not _DYNAMIC_HELPERS:
        def get_proximityReportMode(self) -> int:
            """
            Returns the parameter (sensor value, presence or pulse count) returned by the get_currentValue
            function and callbacks.

            @return a value among YProximity.PROXIMITYREPORTMODE_NUMERIC,
            YProximity.PROXIMITYREPORTMODE_PRESENCE and YProximity.PROXIMITYREPORTMODE_PULSECOUNT corresponding
            to the parameter (sensor value, presence or pulse count) returned by the get_currentValue function and callbacks

            On failure, throws an exception or returns YProximity.PROXIMITYREPORTMODE_INVALID.
            """
            return self._run(self._aio.get_proximityReportMode())

    if not _DYNAMIC_HELPERS:
        def set_proximityReportMode(self, newval: int) -> int:
            """
            Changes the  parameter  type (sensor value, presence or pulse count) returned by the
            get_currentValue function and callbacks.
            The edge count value is limited to the 6 lowest digits. For values greater than one million, use
            get_pulseCounter().
            Remember to call the saveToFlash() method of the module if the modification must be kept.

            @param newval : a value among YProximity.PROXIMITYREPORTMODE_NUMERIC,
            YProximity.PROXIMITYREPORTMODE_PRESENCE and YProximity.PROXIMITYREPORTMODE_PULSECOUNT corresponding
            to the  parameter  type (sensor value, presence or pulse count) returned by the get_currentValue
            function and callbacks

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_proximityReportMode(newval))

    @classmethod
    def FindProximity(cls, func: str) -> YProximity:
        """
        Retrieves a proximity sensor for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the proximity sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YProximity.isOnline() to test if the proximity sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a proximity sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the proximity sensor, for instance
                YPROXIM1.proximity1.

        @return a YProximity object allowing you to drive the proximity sensor.
        """
        return cls._proxy(cls, YProximity_aio.FindProximity(func))

    @classmethod
    def FindProximityInContext(cls, yctx: YAPIContext, func: str) -> YProximity:
        """
        Retrieves a proximity sensor for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the proximity sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YProximity.isOnline() to test if the proximity sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a proximity sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the proximity sensor, for instance
                YPROXIM1.proximity1.

        @return a YProximity object allowing you to drive the proximity sensor.
        """
        return cls._proxy(cls, YProximity_aio.FindProximityInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YProximityValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _IS_MICROPYTHON:
        def registerTimedReportCallback(self, callback: YProximityTimedReportCallback) -> int:
            """
            Registers the callback function that is invoked on every periodic timed notification.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and an YMeasure object describing
                    the new advertised value.
            @noreturn
            """
            return super().registerTimedReportCallback(callback)

    if not _DYNAMIC_HELPERS:
        def resetCounter(self) -> int:
            """
            Resets the pulse counter value as well as its timer.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.resetCounter())

    # --- (end of YProximity implementation)

